@@ -1,10 +1,10 @@
 /*
- *   $Id: device-common.c,v 1.10 2009/06/19 07:28:06 psavola Exp $
+ *   $Id: device-common.c,v 1.11 2010/12/14 11:58:21 psavola Exp $
  *
  *   Authors:
- *    Lars Fenneberg		<lf@elemental.net>	 
+ *    Lars Fenneberg		<lf@elemental.net>
  *
- *   This software is Copyright 1996,1997 by the above mentioned author(s), 
+ *   This software is Copyright 1996,1997 by the above mentioned author(s),
  *   All Rights Reserved.
  *
  *   The license which is distributed with this software in the file COPYRIGHT
@@ -13,23 +13,23 @@
  *
  */
 
-#include <config.h>
-#include <includes.h>
-#include <radvd.h>
-#include <defaults.h>
+#include "config.h"
+#include "includes.h"
+#include "radvd.h"
+#include "defaults.h"
 
 int
 check_device(int sock, struct Interface *iface)
 {
 	struct ifreq	ifr;
-	
+
 	strncpy(ifr.ifr_name, iface->Name, IFNAMSIZ-1);
 	ifr.ifr_name[IFNAMSIZ-1] = '\0';
 
 	if (ioctl(sock, SIOCGIFFLAGS, &ifr) < 0)
 	{
 		if (!iface->IgnoreIfMissing)
-			flog(LOG_ERR, "ioctl(SIOCGIFFLAGS) failed for %s: %s", 
+			flog(LOG_ERR, "ioctl(SIOCGIFFLAGS) failed for %s: %s",
 				iface->Name, strerror(errno));
 		return (-1);
 	}
