@@ -1,11 +1,11 @@
 /*
- *   $Id: recv.c,v 1.12 2010/01/28 13:34:26 psavola Exp $
+ *   $Id: recv.c,v 1.13 2010/12/14 11:58:21 psavola Exp $
  *
  *   Authors:
  *    Pedro Roque		<roque@di.fc.ul.pt>
- *    Lars Fenneberg		<lf@elemental.net>	 
+ *    Lars Fenneberg		<lf@elemental.net>
  *
- *   This software is Copyright 1996,1997 by the above mentioned author(s), 
+ *   This software is Copyright 1996,1997 by the above mentioned author(s),
  *   All Rights Reserved.
  *
  *   The license which is distributed with this software in the file COPYRIGHT
@@ -14,14 +14,14 @@
  *
  */
 
-#include <config.h>
-#include <includes.h>
-#include <radvd.h>
+#include "config.h"
+#include "includes.h"
+#include "radvd.h"
 
 int
 recv_rs_ra(int sock, unsigned char *msg, struct sockaddr_in6 *addr,
                  struct in6_pktinfo **pkt_info, int *hoplimit)
-{                 
+{
 	struct msghdr mhdr;
 	struct cmsghdr *cmsg;
 	struct iovec iov;
@@ -47,7 +47,7 @@ recv_rs_ra(int sock, unsigned char *msg, struct sockaddr_in6 *addr,
 	{
 		if (errno != EINTR)
 			flog(LOG_ERR, "select: %s", strerror(errno));
-			
+
 		return -1;
 	}
 
@@ -68,7 +68,7 @@ recv_rs_ra(int sock, unsigned char *msg, struct sockaddr_in6 *addr,
 	{
 		if (errno != EINTR)
 			flog(LOG_ERR, "recvmsg: %s", strerror(errno));
-			
+
 		return len;
 	}
 
@@ -78,13 +78,13 @@ recv_rs_ra(int sock, unsigned char *msg, struct sockaddr_in6 *addr,
 	{
           if (cmsg->cmsg_level != IPPROTO_IPV6)
           	continue;
-          
+
           switch(cmsg->cmsg_type)
           {
 #ifdef IPV6_HOPLIMIT
               case IPV6_HOPLIMIT:
-                if ((cmsg->cmsg_len == CMSG_LEN(sizeof(int))) && 
-                    (*(int *)CMSG_DATA(cmsg) >= 0) && 
+                if ((cmsg->cmsg_len == CMSG_LEN(sizeof(int))) &&
+                    (*(int *)CMSG_DATA(cmsg) >= 0) &&
                     (*(int *)CMSG_DATA(cmsg) < 256))
                 {
                   *hoplimit = *(int *)CMSG_DATA(cmsg);
@@ -93,8 +93,8 @@ recv_rs_ra(int sock, unsigned char *msg, struct sockaddr_in6 *addr,
                 {
                   flog(LOG_ERR, "received a bogus IPV6_HOPLIMIT from the kernel! len=%d, data=%d",
                   	cmsg->cmsg_len, *(int *)CMSG_DATA(cmsg));
-                  return (-1);	
-                }  
+                  return (-1);
+                }
                 break;
 #endif /* IPV6_HOPLIMIT */
               case IPV6_PKTINFO:
@@ -105,14 +105,14 @@ recv_rs_ra(int sock, unsigned char *msg, struct sockaddr_in6 *addr,
                 }
                 else
                 {
-                  flog(LOG_ERR, "received a bogus IPV6_PKTINFO from the kernel! len=%d, index=%d", 
+                  flog(LOG_ERR, "received a bogus IPV6_PKTINFO from the kernel! len=%d, index=%d",
                   	cmsg->cmsg_len, ((struct in6_pktinfo *)CMSG_DATA(cmsg))->ipi6_ifindex);
                   return (-1);
-                } 
+                }
                 break;
           }
 	}
-	
+
 	dlog(LOG_DEBUG, 4, "recvmsg len=%d", len);
 
 	return len;
